/*
	get configuration to file
*/

#include <stdio.h>
#include <unistd.h>
#include <string.h> /* for strncpy */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <net/if.h>
#include <arpa/inet.h>

#include "include/DefInputS.h"
#include "include/DefInputA.h"
#include "include/DefInputD.h"
#include "include/DefOutput.h"
#include "include/DefSpecial.h"
#include "include/DefTimer.h"
#include "include/DefCommon.h"
#include "include/DefConfig.h"
#include "include/DefIo.h"
#include "include/DefCommFunc.h"
#include "include/DefSwType.h"
#include "include/DefRadio.h"
#include "include/DefAudio.h"
#include "include/DefLightout.h"

//common
char *data;			//data from web page
char dataTemp[100];
int16_t intTemp;
char fileName[100];
FILE *fp;

CONFIG_WEB_DATA cwData;		//web config data


INPUT_STANDARD_DATA 	isData;			//standard input data
INPUT_AUXILIARY_DATA 	iaData;    		//auxiliary input data
INPUT_DTMF_DATA 	idData;    		//DTMF input data
OUTPUT_DATA 		opData;			//output data
TIMER_DATA 		trData;			//timer data
SPECT_FUNCTION_DATA 	sfData;			//special function data
IO_DEF_DATA		ioData[MAX_NUM_IO];	//io pins data
COMM_FUNC_DATA		commData;		//comm function data
SWITCH_TYPE_FUNC_DATA	swtypeData;		//switch type function data
RADIO_FUNC_DATA		radioData;		//radio function data
AUDIO_DATA		aoData;			//audio output data
SPECT_LIGHTOUT_DATA	lightoutData;		//lightout data

void getStandardInput();
void getAuxiliaryInput();
void getDTMFInput();
void getOutput();
void getTimer();
void getSpecial();
void getIO();
void printIO();
void getCommFunc();
void getSwtypeFunc();
void getRadioFunc();
void getAudio();
void getLightout();

int main()
{
	memset(dataTemp, 0x0, sizeof(dataTemp));
	memset(&cwData, 0x0, sizeof(CONFIG_WEB_DATA));
	memset(&isData, 0x0, sizeof(INPUT_STANDARD_DATA));
	memset(&iaData, 0x0, sizeof(INPUT_AUXILIARY_DATA));
	memset(&idData, 0x0, sizeof(INPUT_DTMF_DATA));
	memset(&opData, 0x0, sizeof(OUTPUT_DATA));
	memset(&trData, 0x0, sizeof(TIMER_DATA));
	memset(&sfData, 0x0, sizeof(SPECT_FUNCTION_DATA));
	memset(&ioData, 0x0, sizeof(IO_DEF_DATA) * MAX_NUM_IO);
	memset(&commData, 0x0, sizeof(COMM_FUNC_DATA));
	memset(&swtypeData, 0x0, sizeof(SWITCH_TYPE_FUNC_DATA));
	memset(&radioData, 0x0, sizeof(RADIO_FUNC_DATA));
	memset(&aoData, 0x0, sizeof(AUDIO_DATA));

	printf("Content-Type: text/html\n\n");

	data = (char *)getenv("QUERY_STRING");
	if(data == NULL)
	{
		printf("fail");
	}
	else
	{
		sscanf(data, "type=%1s", dataTemp);
		if(dataTemp[0] == 'S')
		{
			//standard inputs
			getStandardInput();
			printf("p1=%c|p2=%d|p3=%c|p4=%c|p5=%c|p6=%c|p7=%c|p8=%c|p9=%d|p10=%c|p11=%d|p12=%c|p13=%d|p14=%c|p15=%d|p16=%c|p17=%d|p18=%c|p19=%d|p20=%c|p21=%d|p22=%f|p23=%f|p24=%f|p25=%f",
		                isData.sps, isData.dbgi, isData.btntoggle, isData.btnnorrev,
                		isData.pressure, isData.distimern, isData.distimert, isData.distimerr,
		                isData.distime, isData.disthrowm, isData.disthrowmt, isData.disthrowd, isData.disthrowdt,
				isData.disthrowp1, isData.disthrowp1t,
				isData.disthrowp2, isData.disthrowp2t,
				isData.disthrowp3, isData.disthrowp3t,
				isData.disthrowp4, isData.disthrowp4t,
				isData.track_id1, isData.track_id2, isData.track_id3, isData.track_id4
			);
		}
		else if(dataTemp[0] == 'A')
		{
			//auxiliary inputs
			getAuxiliaryInput();
			printf("p1=%c|p2=%d|p3=%c|p4=%c|p5=%c|p6=%c|p7=%c|p8=%c|p9=%d",
		                iaData.aps, iaData.dbgi, iaData.btnnormal, iaData.btntoggle,
                		iaData.btnreverse, iaData.distimern, iaData.distimert, iaData.distimerr,
		                iaData.distime);
		}
		else if(dataTemp[0] == 'D')
		{
			//dtmf inputs
			getDTMFInput();
			printf("p3=%c|p4=%c|p5=%c|p6=%c|p7=%c|p8=%c|p9=%d",
		                idData.btnnormal, idData.btntoggle,
                		idData.btnreverse, idData.distimern, idData.distimert, idData.distimerr,
		                idData.distime);
		}
		else if(dataTemp[0] == 'O')
		{
			getOutput();
			printf("p1=%c|p2=%c|p3=%d|p4=%d|p5=%c|p6=%d|p7=%c|p8=%c|p9=%c|p10=%c|p11=%d|p12=%c",
		                opData.lights, opData.pinf, opData.duration, opData.flashrate,
                		opData.strobe, opData.stroberate, opData.gapthrow, opData.gapphase,
				opData.strobe_gapped, opData.strobe_out_phase,
				opData.track_pos_rate, opData.track_off
			);
		}
		else if(dataTemp[0] == 'T')
		{
			getTimer();
			printf("p1=%c|p2=%c|p3=%c|p4=%d|p5=%lf|p6=%lf|p7=%lf|p8=%d|p9=%d|p10=%d|p11=%d",
		                trData.srai, trData.srdi, trData.srf, trData.srtime,
                		trData.psdebounce, trData.psbreaktime, trData.psintertime,
                		trData.inputdebounce, trData.maxrtime, trData.dvcdtime,
				trData.ldttime
			);
		}
		else if(dataTemp[0] == 'L')
		{
			getSpecial();
			printf("p1=%c|p2=%c|p3=%c|p4=%c|p5=%lf|p6=%lf",
		                sfData.salone, sfData.swstatus, sfData.cpd, sfData.aureturn,
                		sfData.bcaution, sfData.bdisable
			);
		}
		else if(dataTemp[0] == 'P')
		{
			getIO();
			printIO();
			/*printf("p1=%c|p2=%c|p3=%c|p4=%c|p5=%lf|p6=%lf",
		                sfData.salone, sfData.swstatus, sfData.cpd, sfData.aureturn,
                		sfData.bcaution, sfData.bdisable
			);*/
		}
		else if(dataTemp[0] == 'C')
		{
			getCommFunc();
			printf("p1=%c|p2=%d|p3=%c|p4=%c|p5=%d|p6=%d|p7=%d|p8=%d",
				commData.type, commData.baudRate, commData.modeRep, commData.modeTra,
				commData.nodeID, commData.netID, commData.railroadID, commData.yardID
			);
		}
		else if(dataTemp[0] == 'B')
		{
			getSwtypeFunc();
			printf("p1=%c", swtypeData.type);
		}
		else if(dataTemp[0] == 'R')
		{
			getRadioFunc();
			printf("p1=%c|p9=%c|p10=%d|p11=%lf|p12=%d|p13=%d|p14=%c|p15=%d|p16=%d",
			radioData.enable, radioData.LPO,
			radioData.TRBT, radioData.TTID, radioData.TRCL,
			radioData.TPCL, radioData.enableAC, radioData.ACTBP,
			radioData.ACRT);
		}
		else if(dataTemp[0] == 'U')
		{
			getAudio();
			printf("p1=%d|p2=%d|p3=%d|p4=%d|p5=%d|p6=%d",
				aoData.dur_dtmf_nor, aoData.dur_dtmf_rev, aoData.dur_dtmf_gap,
				aoData.dur_aux_1, aoData.dur_aux_2, aoData.dur_aux_3);
		}
		else if(dataTemp[0] == 'V')
		{
			getLightout();
			printf("p1=%c|p2=%d", lightoutData.enable, lightoutData.caution);
		}
	}

	//printf("%s\n", "test123");

	return 0;
}

/*******************************************************************************
  Class :
  Function Name : getStandardInput
  Input         :
  Output        :
  purpose       : get standard input config
  Note          :
*******************************************************************************/
void getStandardInput()
{
	sprintf(fileName,"/home/flexyard/flexyard/sysconfig/WebConfig.dat");

	if((fp = fopen (fileName,"r")) != NULL)
	{
		fread(&cwData, sizeof(CONFIG_WEB_DATA), 1, fp);
		fclose(fp);
	}

	memcpy(&isData, &cwData.isData, sizeof(INPUT_STANDARD_DATA));
}


/*******************************************************************************
  Class :
  Function Name :
  Input         :
  Output        :
  purpose       :
  Note          :
*******************************************************************************/
void getAuxiliaryInput()
{
	sprintf(fileName,"/home/flexyard/flexyard/sysconfig/WebConfig.dat");

	if((fp = fopen (fileName,"r")) != NULL)
	{
		fread(&cwData, sizeof(CONFIG_WEB_DATA), 1, fp);
		fclose(fp);
	}

	memcpy(&iaData, &cwData.iaData, sizeof(INPUT_AUXILIARY_DATA));
	//printf("cwData.isData.distime=%d\n", cwData.isData.distime);
}

/*******************************************************************************
  Class :
  Function Name :
  Input         :
  Output        :
  purpose       :
  Note          :
*******************************************************************************/
void getDTMFInput()
{
	sprintf(fileName,"/home/flexyard/flexyard/sysconfig/WebConfig.dat");

	if((fp = fopen (fileName,"r")) != NULL)
	{
		fread(&cwData, sizeof(CONFIG_WEB_DATA), 1, fp);
		fclose(fp);
	}

	memcpy(&idData, &cwData.idData, sizeof(INPUT_DTMF_DATA));
	//printf("cwData.isData.distime=%d\n", cwData.isData.distime);
}

/*******************************************************************************
  Class :
  Function Name :
  Input         :
  Output        :
  purpose       :
  Note          :
*******************************************************************************/
void getOutput()
{
	sprintf(fileName,"/home/flexyard/flexyard/sysconfig/WebConfig.dat");

	if((fp = fopen (fileName,"r")) != NULL)
	{
		fread(&cwData, sizeof(CONFIG_WEB_DATA), 1, fp);
		fclose(fp);
	}

	memcpy(&opData, &cwData.opData, sizeof(OUTPUT_DATA));
	//printf("cwData.isData.distime=%d\n", cwData.isData.distime);
}

/*******************************************************************************
  Class :
  Function Name :
  Input         :
  Output        :
  purpose       :
  Note          :
*******************************************************************************/
void getTimer()
{
	sprintf(fileName,"/home/flexyard/flexyard/sysconfig/WebConfig.dat");

	if((fp = fopen (fileName,"r")) != NULL)
	{
		fread(&cwData, sizeof(CONFIG_WEB_DATA), 1, fp);
		fclose(fp);
	}

	memcpy(&trData, &cwData.trData, sizeof(TIMER_DATA));
	//printf("cwData.isData.distime=%d\n", cwData.isData.distime);
}

/*******************************************************************************
  Class :
  Function Name :
  Input         :
  Output        :
  purpose       :
  Note          :
*******************************************************************************/
void getSpecial()
{
	sprintf(fileName,"/home/flexyard/flexyard/sysconfig/WebConfig.dat");

	if((fp = fopen (fileName,"r")) != NULL)
	{
		fread(&cwData, sizeof(CONFIG_WEB_DATA), 1, fp);
		fclose(fp);
	}

	memcpy(&sfData, &cwData.sfData, sizeof(SPECT_FUNCTION_DATA));
}

/*******************************************************************************
  Class :
  Function Name :
  Input         :
  Output        :
  purpose       :
  Note          :
*******************************************************************************/
void getIO()
{
	sprintf(fileName,"/home/flexyard/flexyard/sysconfig/WebConfig.dat");

	if((fp = fopen (fileName,"r")) != NULL)
	{
		fread(&cwData, sizeof(CONFIG_WEB_DATA), 1, fp);
		fclose(fp);
	}

	memcpy(&ioData, &cwData.ioData, sizeof(IO_DEF_DATA) * MAX_NUM_IO);
}

/*******************************************************************************
  Class :
  Function Name :
  Input         :
  Output        :
  purpose       :
  Note          :
*******************************************************************************/
void printIO()
{
	int i;
	char content[50000];
	char tempStr[50];
	memset(content, 0x0, sizeof(content));

	for(i = 0; i < MAX_NUM_IO; i++)
	{
                memset(tempStr, 0x0, sizeof(tempStr));
                sprintf(tempStr, "%d||", ioData[i].pinNo);
                strcat(content, tempStr);

		memset(tempStr, 0x0, sizeof(tempStr));
                if(strlen(ioData[i].devName) == 0)
                {
                        sprintf(ioData[i].devName, "%s", "none");
                }
                sprintf(tempStr, "%s||", ioData[i].devName);
                strcat(content, tempStr);

                memset(tempStr, 0x0, sizeof(tempStr));
                sprintf(tempStr, "%d||", ioData[i].devID);
                strcat(content, tempStr);

		memset(tempStr, 0x0, sizeof(tempStr));
                if(ioData[i].devType == '\0')
                {
                        ioData[i].devType = '*';
                }
                sprintf(tempStr, "%c||", ioData[i].devType);
                strcat(content, tempStr);

                memset(tempStr, 0x0, sizeof(tempStr));
                if(ioData[i].devSubType == '\0')
                {
                        ioData[i].devSubType = '*';
                }
                sprintf(tempStr, "%c||", ioData[i].devSubType);
                strcat(content, tempStr);

		memset(tempStr, 0x0, sizeof(tempStr));
                if(ioData[i].devHighLow == '\0')
                {
                        ioData[i].devHighLow = '*';
                }
                sprintf(tempStr, "%c||", ioData[i].devHighLow);
                strcat(content, tempStr);

                memset(tempStr, 0x0, sizeof(tempStr));
                if(ioData[i].devIoType == '\0')
                {
                        ioData[i].devIoType = '*';
                }
                sprintf(tempStr, "%c||", ioData[i].devIoType);
                strcat(content, tempStr);
	}

	/*for(i = 0; i < MAX_NUM_IO; i++)
	{
		//1-3
		memset(tempStr, 0x0, sizeof(tempStr));
                if(strlen(ioData[i].devName) == 0)
                {
                        sprintf(ioData[i].devName, "%s", "none");
                }
                sprintf(tempStr, "%s||", ioData[i].devName);
                strcat(content, tempStr);

                memset(tempStr, 0x0, sizeof(tempStr));
                sprintf(tempStr, "%d||", ioData[i].devID);
                strcat(content, tempStr);

                memset(tempStr, 0x0, sizeof(tempStr));
                sprintf(tempStr, "%d||", ioData[i].devSubID);
                strcat(content, tempStr);

		//4-6
		memset(tempStr, 0x0, sizeof(tempStr));
                if(ioData[i].devType == '\0')
                {
                        ioData[i].devType = '*';
                }
                sprintf(tempStr, "%c||", ioData[i].devType);
                strcat(content, tempStr);

                memset(tempStr, 0x0, sizeof(tempStr));
                if(ioData[i].devSubType == '\0')
                {
                        ioData[i].devSubType = '*';
                }
                sprintf(tempStr, "%c||", ioData[i].devSubType);
                strcat(content, tempStr);

                memset(tempStr, 0x0, sizeof(tempStr));
                sprintf(tempStr, "%d||", ioData[i].devNode);
                strcat(content, tempStr);

		//7-9
		memset(tempStr, 0x0, sizeof(tempStr));
                sprintf(tempStr, "%d||", ioData[i].devBoard);
                strcat(content, tempStr);

                memset(tempStr, 0x0, sizeof(tempStr));
                sprintf(tempStr, "%d||", ioData[i].devByte);
                strcat(content, tempStr);

                memset(tempStr, 0x0, sizeof(tempStr));
                sprintf(tempStr, "%d||", ioData[i].devBit);
                strcat(content, tempStr);

		//10-12
		memset(tempStr, 0x0, sizeof(tempStr));
                if(ioData[i].devHighLow == '\0')
                {
                        ioData[i].devHighLow = '*';
                }
                sprintf(tempStr, "%c||", ioData[i].devHighLow);
                strcat(content, tempStr);

                memset(tempStr, 0x0, sizeof(tempStr));
                if(ioData[i].devIoType == '\0')
                {
                        ioData[i].devIoType = '*';
                }
                sprintf(tempStr, "%c||", ioData[i].devIoType);
                strcat(content, tempStr);

                memset(tempStr, 0x0, sizeof(tempStr));
                if(ioData[i].devIoSubType == '\0')
                {
                        ioData[i].devIoSubType = '*';
                }
                sprintf(tempStr, "%c||", ioData[i].devIoSubType);
                strcat(content, tempStr);

		//13-15
		memset(tempStr, 0x0, sizeof(tempStr));
                if(ioData[i].devIoFamily == '\0')
                {
                        ioData[i].devIoFamily = '*';
                }
                sprintf(tempStr, "%c||", ioData[i].devIoFamily);
                strcat(content, tempStr);

                memset(tempStr, 0x0, sizeof(tempStr));
                sprintf(tempStr, "%d||", ioData[i].devTime);
                strcat(content, tempStr);

                memset(tempStr, 0x0, sizeof(tempStr));
                sprintf(tempStr, "%d||", ioData[i].devSlowTime);
                strcat(content, tempStr);

		//16-18
		memset(tempStr, 0x0, sizeof(tempStr));
                sprintf(tempStr, "%d||", ioData[i].serialNo);
                strcat(content, tempStr);

                memset(tempStr, 0x0, sizeof(tempStr));
                if(ioData[i].devMade == '\0')
                {
                        ioData[i].devMade = '*';
                }
                sprintf(tempStr, "%c||", ioData[i].devMade);
                strcat(content, tempStr);

                //memset(tempStr, 0x0, sizeof(tempStr));
                //sprintf(tempStr, "%d||", ioData[i].mode);
                //strcat(content, tempStr);

		//19-21
		//memset(tempStr, 0x0, sizeof(tempStr));
                //sprintf(tempStr, "%d||", ioData[i].board);
                //strcat(content, tempStr);

                //memset(tempStr, 0x0, sizeof(tempStr));
                //sprintf(tempStr, "%d||", ioData[i].input);
                //strcat(content, tempStr);

                //memset(tempStr, 0x0, sizeof(tempStr));
                //sprintf(tempStr, "%d||", ioData[i].channel);
                //strcat(content, tempStr);

		//printf("ioData[%d] = %s,%d,%d,   %c,%c,%d,   %d,%d,%d,   %c,%c,%c,   %c,%d,%d,   %d,%c,%d,   %d,%d,%d \n", i,
                //        ioData[i].devName, ioData[i].devID, ioData[i].devSubID,
                //        ioData[i].devType, ioData[i].devSubType, ioData[i].devNode,
                //        ioData[i].devBoard, ioData[i].devByte, ioData[i].devBit,
                //        ioData[i].devHighLow, ioData[i].devIoType, ioData[i].devIoSubType,
                //        ioData[i].devIoFamily, ioData[i].devTime, ioData[i].devSlowTime,
                //        ioData[i].serialNo, ioData[i].devMade, ioData[i].mode,
                //        ioData[i].board, ioData[i].input, ioData[i].channel
                //        );
	}*/

        printf(content);
}

/*******************************************************************************
  Class :
  Function Name :
  Input         :
  Output        :
  purpose       :
  Note          :
*******************************************************************************/
void getCommFunc()
{
	sprintf(fileName,"/home/flexyard/flexyard/sysconfig/WebConfig.dat");

	if((fp = fopen (fileName,"r")) != NULL)
	{
		fread(&cwData, sizeof(CONFIG_WEB_DATA), 1, fp);
		fclose(fp);
	}

	memcpy(&commData, &cwData.commData, sizeof(COMM_FUNC_DATA));
}

/*******************************************************************************
  Class :
  Function Name :
  Input         :
  Output        :
  purpose       :
  Note          :
*******************************************************************************/
void getSwtypeFunc()
{
	sprintf(fileName,"/home/flexyard/flexyard/sysconfig/WebConfig.dat");

	if((fp = fopen (fileName,"r")) != NULL)
	{
		fread(&cwData, sizeof(CONFIG_WEB_DATA), 1, fp);
		fclose(fp);
	}

	memcpy(&swtypeData, &cwData.swtypeData, sizeof(SWITCH_TYPE_FUNC_DATA));
}

/*******************************************************************************
  Class :
  Function Name :
  Input         :
  Output        :
  purpose       :
  Note          :
*******************************************************************************/
void getRadioFunc()
{
	sprintf(fileName,"/home/flexyard/flexyard/sysconfig/WebConfig.dat");

	if((fp = fopen (fileName,"r")) != NULL)
	{
		fread(&cwData, sizeof(CONFIG_WEB_DATA), 1, fp);
		fclose(fp);
	}

	memcpy(&radioData, &cwData.radioData, sizeof(RADIO_FUNC_DATA));
}

/*******************************************************************************
  Class :
  Function Name :
  Input         :
  Output        :
  purpose       :
  Note          :
*******************************************************************************/
void getAudio()
{
	sprintf(fileName,"/home/flexyard/flexyard/sysconfig/WebConfig.dat");

	if((fp = fopen (fileName,"r")) != NULL)
	{
		fread(&cwData, sizeof(CONFIG_WEB_DATA), 1, fp);
		fclose(fp);
	}

	memcpy(&aoData, &cwData.aoData, sizeof(AUDIO_DATA));
}

/*******************************************************************************
  Class :
  Function Name :
  Input         :
  Output        :
  purpose       :
  Note          :
*******************************************************************************/
void getLightout()
{
	sprintf(fileName,"/home/flexyard/flexyard/sysconfig/lightout.dat");

	if((fp = fopen (fileName,"r")) != NULL)
	{
		fread(&lightoutData, sizeof(SPECT_LIGHTOUT_DATA), 1, fp);
		fclose(fp);
	}
}

